#include "cora_fit.h"
#include <stdio.h>

static int copy_from_llambda ()
{
  int i, k = 0;

  for (i = 0; i < nlines; ++i) 
    {
      if (fit_wave[i] == TRUE && fit_lock[i] == FALSE)
	{
	  mP.vec[k] = llambda.vec[i];
	  ++k;
	}
    }
  return k;
}

static int copy_from_lsigma (int start)
{
  int i, k = 0;

  for (i = 0; i < nlines; ++i) 
    {
      if (fit_sigm[i] == TRUE)
	{
	  mP.vec[k+start] = lsigma.vec[i];
	  ++k;
	}
    }
  return k;
}

/* This function is called only once (from main()); it sets upper/lower
 * limits, and copies those lambda/sigma that are to be iterated.
 */ 
void borders()
{
  /*  define borders for penalties */
  
  int i=0;
  int copied;
  

  /*------- all ---------*/

  vec_alloc (&lP,     2*nlines);
  vec_alloc (&uP,     2*nlines);
  vec_alloc (&startP, 2*nlines);
  
  
  lP.vec[0]          = llambda.vec[0]-dlam;
  startP.vec[0]      = llambda.vec[0];
  uP.vec[0]          = llambda.vec[0]+dlam;
  lP.vec[nlines]     = lsigma.vec[0]-dsig;
  startP.vec[nlines] = lsigma.vec[0];
  uP.vec[nlines]     = lsigma.vec[0]+dsig;

  for(i=1; i < nlines; ++i)
    {
      lP.vec[i]            = lP.vec[i-1]+steplam.vec[i-1];
      uP.vec[i]            = lP.vec[i]+2.*dlam;
      lP.vec[i+nlines]     = lP.vec[i+nlines-1]+stepsig.vec[i-1];
      uP.vec[i+nlines]     = lP.vec[i+nlines]+2.*dsig;
      startP.vec[i]	   = llambda.vec[i];
      startP.vec[i+nlines] = lsigma.vec[i];
    }


  /*------- all ---------*/

  if (isigit > 0 && ilamit > 0 && idlam == 0 && idsig == 0)
    {
      vec_alloc(&mP, 2 * nlines);
      funcname = func_lik;

      copied  = copy_from_llambda();
      imaxfit = copy_from_lsigma(copied) + copied;
      mP.len  = imaxfit;

      /*
      for(i=0; i < 2*nlines; ++i) 
	mP.vec[i]=startP.vec[i];
      */

      if (verbose) 
	printf("Message      : using function func_lik for minimization\n");
    }

  /*------- all s_const ---------*/

  else  if (isigit > 0 && ilamit > 0 && idlam == 0 && idsig > 0)
    {
      vec_alloc(&mP,nlines+1);
      funcname = func_sigclik;

      imaxfit   = copy_from_llambda(0) + 1;
      i=0;
      while (fit_sigm[i] == FALSE) ++i;
      mP.vec[imaxfit-1] = lsigma.vec[i];
      mP.len    = imaxfit;

      if (verbose)
        printf("Message      : using function func_sigclik for minimization\n");
    }

  /*------- all l_const ---------*/

  else  if (isigit > 0 && ilamit > 0 && idlam > 0 && idsig == 0)
    {
      vec_alloc(&mP,nlines+1);
      funcname = func_lamclik;

      mP.vec[0] = startP.vec[0];
      imaxfit   = copy_from_lsigma(1) + 1;
      mP.len    = imaxfit;

      if (verbose) 
	printf("Message      : using function func_lamclik for minimization\n");
    }

  /*------- all l/s_const ---------*/

  else  if (isigit > 0 && ilamit > 0 && idlam > 0 && idsig > 0)
    {
      vec_alloc(&mP,2);
      funcname = func_clik;

      mP.vec[0] = startP.vec[0];
      mP.len    = 2;

      i=0;
      while (fit_sigm[i] != TRUE) ++i;
      mP.vec[1] = startP.vec[nlines+i];

      if (verbose)
        printf("Message      : using function func_clik for minimization\n");
    }

  /*------- lambda ---------*/

  else  if (isigit == 0 && ilamit > 0 && idlam == 0)
    {
      vec_alloc(&mP,nlines);
      funcname = func_lamlik;

      copied  = copy_from_llambda();
      imaxfit = copied;
      mP.len  = imaxfit;
      /*
      for(i=0; i<nlines; ++i) 
	mP.vec[i]=startP.vec[i];
      */

      if (verbose) 
	printf("Message      : using function func_lamlik for minimization\n");
    }

  /*------- lambda l_const ---------*/

  else  if (isigit == 0 && ilamit > 0 && idlam > 0)
    {
      vec_alloc(&mP,1);
      funcname = func_clamlik;

      mP.vec[0] = startP.vec[0];
      imaxfit   = 1;
      mP.len    = imaxfit;

      if (verbose) 
	printf("Message      : using function func_clamlik "\
	       "for minimization\n");
    }

 /*------- sigma ---------*/

  else  if (isigit > 0 && ilamit == 0 && idsig == 0)
    { 
      vec_alloc(&mP,nlines);
      funcname = func_siglik;

      copied  = copy_from_lsigma(0);
      imaxfit = copied;
      mP.len  = imaxfit;
      /*
      for (i = 0; i < nlines; ++i) 
	mP.vec[i] = startP.vec[i+nlines];
      */

      if (verbose) 
	printf("Message      : using function func_siglik for minimization\n");
    } 

 /*----- sigma const ----*/

  else  if (isigit > 0 && ilamit == 0 && idsig > 0)
    {
      vec_alloc(&mP,1);
      funcname = func_csiglik;

      i=0;
      while (fit_sigm[i] != TRUE) ++i;
      mP.len  = 1;
      mP.vec[0] = startP.vec[nlines+i];
      /*
      for (i = 0; i < nlines; ++i)
        mP.vec[i] = startP.vec[i+nlines];
      */

      if (verbose)
        printf("Message      : using function func_csiglik for minimization\n");
    }

  /*------- nothing ---------*/

  else 
    {
      if (total(&f) == 0 ) fprintf (stderr, "*  Warning  *: borders(): nothing to iterate.\n");
      vec_alloc(&mP,2*nlines);
      funcname = func_lik;


      copied  = copy_from_llambda();
      imaxfit = copy_from_lsigma(copied) + copied;
      mP.len  = imaxfit;
      /*
      for(i = 0; i < 2*nlines; ++i)
        mP.vec[i] = startP.vec[i];
      */

      if (verbose) 
	printf("Message      : using function func_lik for minimization\n");
    }
  
  return;
}
